package com.diagbot.facade;

import com.diagbot.dto.DrugInfoDTO;
import com.diagbot.dto.LisDetailDTO;
import com.diagbot.dto.OperationInfoDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.entity.node.LIS;
import com.diagbot.entity.node.LISPack;
import com.diagbot.repository.DeptNode;
import com.diagbot.repository.DeptRepository;
import com.diagbot.repository.ICDDiseaseNode;
import com.diagbot.repository.ICDDiseaseRepository;
import com.diagbot.repository.LisNode;
import com.diagbot.repository.LisPackNode;
import com.diagbot.repository.LisPackRepository;
import com.diagbot.repository.LisRepository;
import com.diagbot.repository.MedicineNode;
import com.diagbot.repository.MedicineRepository;
import com.diagbot.repository.OperationNode;
import com.diagbot.repository.OperationRepository;
import com.diagbot.repository.PACSNode;
import com.diagbot.repository.PacsRepository;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/12 16:44
 */
@Component
public class RetrievalFacade {

    @Autowired
    LisPackRepository lisPackRepository;

    @Autowired
    LisRepository lisRepository;

    @Autowired
    ICDDiseaseRepository icdDiseaseRepository;

    @Autowired
    PacsRepository pacsRepository;

    @Autowired
    MedicineRepository medicineRepository;

    @Autowired
    OperationRepository operationRepository;

    @Autowired
    DeptRepository deptRepository;

    /**
     * 检索
     *
     * @param retrievalVO
     * @return
     */
    public RetrievalDTO index(RetrievalVO retrievalVO) {
        RetrievalDTO retrievalDTO = new RetrievalDTO();

        switch (retrievalVO.getType()) {
            case 1:
                retrievalDTO.setLisNames(getLisPacks(retrievalVO.getInputStr()));
                break;
            case 2:
                retrievalDTO.setLisDetailNames(getLis(retrievalVO.getInputStr()));
                break;
            case 3:
                retrievalDTO.setPacsNames(getPACSs(retrievalVO.getInputStr()));
                break;
            case 4:
                retrievalDTO.setDiseaseNames(getDiseases(retrievalVO.getInputStr()));
                break;
            case 5:
                retrievalDTO.setDrugNames(getMedicines(retrievalVO.getInputStr()));
                break;
            case 6:
                retrievalDTO.setOperationNames(getOperations(retrievalVO.getInputStr()));
                break;
            case 7:
                retrievalDTO.setDeptNames(getDepts(retrievalVO.getInputStr()));
                break;
        }

        return retrievalDTO;
    }


    /*
     * 获取化验套餐名称列表
     */
    public List<String> getLisPacks(String name) {

        LisPackNode lisPackNode = new LisPackNode();
        List<String> lisPacklist = lisPackNode.getLISPackNameList(name, lisPackRepository);

        return lisPacklist;
    }


    /*
     * 获取化验细项名称列表
     */
    public List<LisDetailDTO> getLis(String name) {

        LisNode lisNode = new LisNode();
        List<LisDetailDTO> lislist = lisNode.getLisDetailList(name, lisRepository);

        return lislist;
    }


    /*
     * 获取辅检名称列表
     */
    public List<String> getPACSs(String name) {

        PACSNode pacsNode = new PACSNode();
        List<String> pacslist = pacsNode.getPACSNameList(name, pacsRepository);

        return pacslist;
    }


    /*
     * 获取疾病名称列表
     */
    public List<String> getDiseases(String name) {

        ICDDiseaseNode diseaseNode = new ICDDiseaseNode();
        List<String> diseaselist = diseaseNode.getDiseaseNameList(name, icdDiseaseRepository);

        return diseaselist;
    }


    /*
     * 获取药品商品名称列表
     */
    public List<DrugInfoDTO> getMedicines(String name) {

        MedicineNode medicineNode = new MedicineNode();
        List<DrugInfoDTO> medslist = medicineNode.getMedicineList(name, medicineRepository);

        return medslist;
    }


    /*
     * 获取手术和操作名称列表
     */
    public List<OperationInfoDTO> getOperations(String name) {

        OperationNode operationNode = new OperationNode();
        List<OperationInfoDTO> opslist = operationNode.getOperationList(name, operationRepository);

        return opslist;
    }


    /*
     * 获取科室名称列表
     */
    public List<String> getDepts(String name) {

        DeptNode deptNode = new DeptNode();
        List<String> deptlist = deptNode.getDeptNameList(name, deptRepository);

        return deptlist;
    }

    /**
     * 医学知识（静态信息）检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    public List<StaticKnowledgeIndexDTO> staticKnowledgeIndex(StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<StaticKnowledgeIndexDTO> retList = Lists.newLinkedList();
        String inputStr = staticKnowledgeIndexVO.getInputStr();
        List<Integer> types = staticKnowledgeIndexVO.getTypes();
        //0-全部、1-诊断、2-药品、3-检验、4-检查、5-手术和操作
        if (types.contains(1)) {
            List<String> diseases = getDiseases(inputStr);
            List<StaticKnowledgeIndexDTO> staticList = Lists.newLinkedList();
            for (String str : diseases) {
                StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                staticDTO.setName(str);
                staticDTO.setType(1);
                staticDTO.setTypeName("诊断");
                staticList.add(staticDTO);
            }
            retList.addAll(staticList);
        } else if (types.contains(2)) {
            List<DrugInfoDTO> items = getMedicines(inputStr);
            List<StaticKnowledgeIndexDTO> staticList = Lists.newLinkedList();
            for (DrugInfoDTO item : items) {
                StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                staticDTO.setName(item.getName());
                staticDTO.setType(2);
                staticDTO.setTypeName("药品");
                staticList.add(staticDTO);
            }
            retList.addAll(staticList);
        } else if (types.contains(3)) {
            List<StaticKnowledgeIndexDTO> staticList = Lists.newLinkedList();
            List<LIS> lislt = lisRepository.findByNameLike("*" + inputStr + "*");
            //套餐显示在前
            for (LIS lis : lislt) {
                if (lis.getLisPacks().size() > 0) {
                    for (LISPack lisPack : lis.getLisPacks()) {
                        StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                        staticDTO.setRetrievalName(lis.getName());
                        staticDTO.setName(lisPack.getName());
                        staticDTO.setType(3);
                        staticDTO.setTypeName("检验");
                        staticList.add(staticDTO);
                    }
                }
            }
            //明细显示在后
            for (LIS lis : lislt) {
                StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                staticDTO.setName(lis.getName());
                staticDTO.setType(3);
                staticDTO.setTypeName("检验");
                staticList.add(staticDTO);
            }
            retList.addAll(staticList);
        } else if (types.contains(4)) {
            List<String> pacSs = getPACSs(inputStr);
            List<StaticKnowledgeIndexDTO> staticList = Lists.newLinkedList();
            for (String str : pacSs) {
                StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                staticDTO.setName(str);
                staticDTO.setType(4);
                staticDTO.setTypeName("检查");
                staticList.add(staticDTO);
            }
            retList.addAll(staticList);
        } else if (types.contains(5)) {
            List<OperationInfoDTO> items = getOperations(inputStr);
            List<StaticKnowledgeIndexDTO> staticList = Lists.newLinkedList();
            for (OperationInfoDTO item : items) {
                StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                staticDTO.setName(item.getName());
                staticDTO.setType(5);
                staticDTO.setTypeName("手术和操作");
                staticList.add(staticDTO);
            }
            retList.addAll(staticList);
        } else {

        }

        return retList;
    }
}